/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Type.h
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#ifndef TYPE_H_
#define TYPE_H_

#include <string>
#include <iostream>

#include <xmltio/xmltio.hpp>

#include "data/common/Timestamp.h"
#include "XmltioType.h"

namespace airobots {

/**
 * Base class for types that are exchanged via xml. Each derived class must
 * contain an implementation of getTemplate that returns a complete template
 * string that can be used to construct an xmltio location. The location must
 * be parsed with the an xpath containing the base tag of the template returned
 * by getBaseTag. So this method must be provided too.
 *
 * To enable reusing existing xml documents as templates (eg. to preserve the
 * dxml:id tag) one needs to be sure that there is no harmful data contained in
 * the (pre-existing) template. This could be the case for a type that manages
 * a list of attributes, eg. like this:
 *
 * <LIST>
 *     <ENTRY a="1"/>
 *     <ENTRY a="2"/>
 *     <ENTRY a="3"/>
 * </LIST>
 *
 * If one reuses this xml structure as a template and the convert method for
 * this type simply appends new entries to the LIST location, then the resulting
 * xml document will contain more list entries than the type knew. For this
 * reason, every type must implement the method <code>sanitizeLocation</code>
 * which ensures that such inconsistencies cannot occur, eg. by removing the
 * pre-existing list entries in the template.
 *
 * Every type must provide a default constructor.
 *
 * @author jwienke
 */
class Type: public XmltioType {

public:

	/**
	 * Exception used to declare that the current BTL object was not parsed from
	 * a xml document.
	 */
	class NoSourceLocationException : public std::exception {
	private:
		std::string message;

	public:
		NoSourceLocationException(std::string message =
				"object has no source document. Possible reasons:"
				"\n * object was not parsed from a xml document"
				"\n * concrete btl type was not implemented properly") {
			this->message = message;
		}

		~NoSourceLocationException() throw() {}

		virtual const char* what() const throw() {
			return message.c_str();
		}
	};

	/**
	 * Default generator name of the generator is not known.
	 */
	const static std::string UNKNOWN_GENERATOR;

	/**
	 * Default source location is not known.
	 */
	const static std::string UNKNOWN_SOURCE_LOCATION;

	Type();

	Type(const std::string &generator);

	Type(const std::string &generator, const uint64_t &created);

	Type(const std::string &generator, const Timestamp &timestamp);

	// ---------- methods to implement in subtypes ------------

	virtual bool operator==(const Type &other) const;

	// and all methods specified in XmltioType.h

	// ---------- base class methods ------------

	/**
	 * Sets legacy parsing
	 *
	 * @param true for types without generator/timestamp
	 */
	void setLegacyParsing(bool legacyParsing);

	/**
	 * Returns true if legacy parsing is on
	 *
	 * @return on/off
	 */
	bool isLegacyParsing() const;

	/**
	 * Returns the generator of this type or <code>UNKNOWN_GENERATOR</code> if
	 * the generator was not set.
	 *
	 * @return string
	 */
	std::string getGenerator() const;

	/**
	 * Sets the generator of this type.
	 *
	 * @param generator name of the generator of this type
	 */
	void setGenerator(const std::string &generator);

	/**
	 * Const getter for the timestamp of this type.
	 *
	 * @return not modifiable timestamp of this type.
	 */
	const Timestamp& getTimestamp() const;

	/**
	 * Getter for the timestamp of this type. This version can be used to change
	 * it.
	 *
	 * @return timestamp of this type
	 */
	Timestamp& getTimestamp();

	/**
	 * Getter for the source xmltio location for this type.
	 *
	 * @return the source xmltio location
	 */
	const xmltio::Location& getSourceLocation() const;

	/**
	 * Getter for the source xmltio location for this type. This version can be used to change
	 * it.
	 *
	 * @return the source xmltio location
	 */
	xmltio::Location& getSourceLocation();

	/**
	 * Sets the source xmltio location for this type.
	 *
	 * @param the source xmltio location
	 */
	void setSourceLocation(const xmltio::Location &loc);

	/**
	 * Setter for the timestamp of this type. This simply replace the current
	 * timestamp with the given one.
	 *
	 * @param timestamp new timestamp
	 */
	void setTimestamp(const Timestamp &timestamp);

	virtual void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, Type &type);

private:

	/**
	 * Should be true for types without generator/timestamp
	 */
	bool legacyParsing;

	/**
	 * Generator component of this type.
	 */
	std::string generator;

	/**
	 * Source xmltio location from where the current BTL object was parsed.
	 */
	xmltio::Location sourceLocation;

	/**
	 * Timestamp.
	 */
	Timestamp timestamp;

};

}

#endif /* TYPE_H_ */
