/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */

#ifndef POIDATA_CPP
#define POIDATA_CPP

#include <string>

#include <boost/shared_ptr.hpp>

#include <xmltio/xmltio.hpp>

#include "../../Type.h"
#include "Face.h"
#include "Legs.h"
#include "Shirt.h"
#include "../../units/UnitConverter.h"

namespace airobots {

class PersonData: public Type {
public:

	static std::string getBaseTag();

	static std::string getTemplate();

	PersonData(std::string generator);
	PersonData();

	float getAngle(AngleUnit angleUnit) const;
	float getDistance(LengthUnit lengthUnit) const;
	bool isFacing() const;
	float getGaze() const;
	int getIdentId() const;
	std::string getIdentName() const;
	bool hasNeighbour() const;
	float getNeighbourDistance(LengthUnit lengthUnit) const;
	int getNeighbourId() const;
	bool isPoi() const;
	bool isTalking() const;
	bool isWalking() const;

	/**
	 * Returns the face percept associated with this person. Pointer may contain
	 * NULL if no face was detected.
	 */
	boost::shared_ptr<Face> getFace() const;

	/**
	 * Tells if there is a face percept associated with this person.
	 */
	bool hasFace() const;

	/**
	 * Returns the legs percept associated with this person. Pointer may contain
	 * NULL if no face was detected.
	 */
	boost::shared_ptr<Legs> getLegs() const;

	/**
	 * Tells if there is a legs percept associated with this person.
	 */
	bool hasLegs() const;

	/**
	 * Returns the shirt percept associated with this person. Pointer may contain
	 * NULL if no shirt was detected.
	 */
	boost::shared_ptr<Shirt> getShirt() const;

	/**
	 * Tells if there is a shirt percept associated with this person.
	 */
	bool hasShirt() const;

	void setAngle(float angle,AngleUnit angleUnit);
	void setDistance(float distance,LengthUnit lengthUnit);
	void setFacing(bool facing);
	void setGaze(float gaze);
	void setIdentId(int identId);
	void setIdentName(std::string identName);
	void setHasNeighbour(bool hasNeighbour);
	void setNeighbourDistance(float neighbourDistance,LengthUnit lengthUnit);
	void setNeighbourId(int neighbourId);
	void setPoi(bool poi);
	void setTalking(bool talking);
	void setWalking(bool walking);

	/**
	 * Sets the face percept associated with this person. May contain a
	 * shared_ptr pointing to NULL to indicate that there is no face percept
	 * for this person.
	 */
	void setFace(boost::shared_ptr<Face> face);

	/**
	 * Sets the legs percept associated with this person. May contain a
	 * shared_ptr pointing to NULL to indicate that there is no legs percept
	 * for this person.
	 */
	void setLegs(boost::shared_ptr<Legs> legs);

	/** 
	 * Sets the shirt percept associated with this person. May contain a
	 * shared_ptr pointing to NULL to indicate that there is no shirt percept
	 * for this person.
	 */
	void setShirt(boost::shared_ptr<Shirt> shirt);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, PersonData &type);

	virtual bool operator==(const PersonData &other) const;

private:

	bool walking;
	bool facing;
	float gaze;
	bool talking;
	float distance;
	float angle;
	bool poi;
	bool grounded;
	std::string identName;
	int identId;
	bool neighbour;
	float neighbourDistance;
	int neighbourId;
	boost::shared_ptr<Face> face;
	boost::shared_ptr<Legs> legs;
	boost::shared_ptr<Shirt> shirt;

};

}

#endif
