/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.geometry;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This class represents a point in 2 dimensions.
 * 
 * @author lziegler
 */
public class Point2D extends Type {

	public enum Scope {
		GLOBAL, LOCAL
	};

	// coordinates in millimeters
	private double x, y;
	private Scope scope = Scope.GLOBAL;

	public Point2D(double x, double y, LengthUnit unit) {
		setX(x, unit);
		setY(y, unit);
	}

	public Point2D(double x, double y, LengthUnit unit, Scope scope) {
		setX(x, unit);
		setY(y, unit);
		this.scope = scope;
	}

	public double getX(LengthUnit unit) {
		return UnitConverter.convert(x, LengthUnit.MILLIMETER, unit);
	}

	public void setX(double x, LengthUnit unit) {
		this.x = UnitConverter.convert(x, unit, LengthUnit.MILLIMETER);
	}

	public double getY(LengthUnit unit) {
		return UnitConverter.convert(y, LengthUnit.MILLIMETER, unit);
	}

	public void setY(double y, LengthUnit unit) {
		this.y = UnitConverter.convert(y, unit, LengthUnit.MILLIMETER);
	}

	public Scope getScope() {
		return scope;
	}

	public void setScope(Scope scope) {
		this.scope = scope;
	}

	public static String getBaseTag() {
		return "POINT2D";
	}

	/**
	 * Constructs a {@link Point2D} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link Point2D} object containing all the information given
	 *         by the {@link Element} object.
	 */
	public static Point2D fromElement(Element objectElement)
			throws ParsingException {

		try {

			if (!objectElement.getLocalName().equals(getBaseTag())) {
				throw new ParsingException("Parsing Error: Base tag "
						+ "for point2d object must be '" + getBaseTag() + "'");
			}

			double x0 = Double.parseDouble(objectElement.getAttribute("x")
					.getValue());
			double y0 = Double.parseDouble(objectElement.getAttribute("y")
					.getValue());

			Scope s0;
			try {
				s0 = Scope.valueOf(objectElement.getAttribute("scope")
						.getValue());
			} catch (IllegalArgumentException e) {
				s0 = Scope.GLOBAL;
			} catch (NullPointerException e) {
				s0 = Scope.GLOBAL;
			}

			return new Point2D(x0, y0, LengthUnit.MILLIMETER, s0);

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present

			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse coordinate values. ", e);
		}
	}

	/**
	 * Serializes the {@link Point2D} object into a given XOM {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {

		parent.addAttribute(new Attribute("x", String
				.valueOf(getX(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("y", String
				.valueOf(getY(LengthUnit.MILLIMETER))));
		parent.addAttribute(new Attribute("scope", String.valueOf(scope)));
	}

	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof Point2D))
				return false;

			Point2D other = (Point2D) obj;

			if (other.getX(LengthUnit.MILLIMETER) != getX(LengthUnit.MILLIMETER))
				return false;
			if (other.getY(LengthUnit.MILLIMETER) != getY(LengthUnit.MILLIMETER))
				return false;
			if (other.getScope() != scope)
				return false;

		} catch (Exception e) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "[" + getBaseTag() + " x=" + x + " y=" + y + " scope=" + scope
				+ "]";
	}
}
