/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.vision3d;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

/**
 * This is the representation of an Point Cloud with grasping point and label.
 * 
 * @author plueckin
 */

public class PointCloudGrasping extends Type {

	public static final String LABEL_TAG = "LABEL";
	public static final String GRASPPNT_TAG = "GRASPING_POINT";
	
	public static String WIDTH_TAG = "WIDTH";
	public static String HEIGHT_TAG = "HEIGHT";
	public static String DEPTH_TAG = "DEPTH";

	private static LengthUnit iLU = LengthUnit.MILLIMETER;

	private String label;
	private Point3D grasppnt;
	private PointCloud convhull;

	private double height;
	private double width;
	private double depth;
	
	public PointCloudGrasping() {
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "POINTCLOUDGRASPING";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// cleanup
		Elements elements = parent.getChildElements(LABEL_TAG);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(GRASPPNT_TAG);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(PointCloud.getBaseTag());
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}

		Element labelName = new Element(LABEL_TAG);
		labelName.addAttribute(new Attribute("value", label));
		parent.appendChild(labelName);
		
		Element grasp = new Element(GRASPPNT_TAG);
		grasppnt.fillInto(grasp);
		parent.appendChild(grasp);

		Element convh = new Element(PointCloud.getBaseTag());
		convhull.fillInto(convh);
		parent.appendChild(convh);
		
		Elements oldHeight = parent.getChildElements(HEIGHT_TAG);
		for (int i = 0; i < oldHeight.size(); i++) {
			parent.removeChild(oldHeight.get(i));
		}
		Element height = new Element(HEIGHT_TAG);
		height.addAttribute(new Attribute("valh", String.valueOf(this.height)));
		parent.appendChild(height);

		Elements oldWidth = parent.getChildElements(WIDTH_TAG);
		for (int i = 0; i < oldWidth.size(); i++) {
			parent.removeChild(oldWidth.get(i));
		}
		Element width = new Element(WIDTH_TAG);
		width.addAttribute(new Attribute("valw", String.valueOf(this.width)));
		parent.appendChild(width);

		Elements oldDepth = parent.getChildElements(DEPTH_TAG);
		for (int i = 0; i < oldDepth.size(); i++) {
			parent.removeChild(oldDepth.get(i));
		}
		Element depth = new Element(DEPTH_TAG);
		depth.addAttribute(new Attribute("vald", String.valueOf(this.depth)));
		parent.appendChild(depth);

	}

	/**
	 * 
	 */
	public static void fromElement(Element dataElement,
			PointCloudGrasping pcpElement) throws ParsingException {

		try {
			if (!dataElement.getLocalName().equals(getBaseTag())) {
				throw new ParsingException("Parsing Error: Base tag "
						+ "for pcpelement object must be '" + getBaseTag()
						+ "'");
			}

			Type.fromElement(dataElement, pcpElement);

			Element grasppnt = dataElement.getFirstChildElement(GRASPPNT_TAG);
			double gpx = Double.parseDouble(grasppnt.getAttribute("x")
					.getValue());
			double gpy = Double.parseDouble(grasppnt.getAttribute("y")
					.getValue());
			double gpz = Double.parseDouble(grasppnt.getAttribute("z")
					.getValue());

			Element height = dataElement.getFirstChildElement(HEIGHT_TAG);
			double h = Double.parseDouble(height.getAttribute("valh")
					.getValue());
			Element width = dataElement.getFirstChildElement(WIDTH_TAG);
			double w = Double
					.parseDouble(width.getAttribute("valw").getValue());
			Element depth = dataElement.getFirstChildElement(DEPTH_TAG);
			double d = Double
					.parseDouble(depth.getAttribute("vald").getValue());
			
			Element convhull = dataElement.getFirstChildElement(PointCloud
					.getBaseTag());
			PointCloud pc = new PointCloud();
			PointCloud.fromElement(convhull, pc);

			Element name = dataElement.getFirstChildElement(LABEL_TAG);
			final String nameLabel = name.getAttribute("value").getValue();
			pcpElement.setLabel(nameLabel);
			pcpElement.setGraspPoint(gpx, gpy, gpz);
			pcpElement.setConvexhull(pc);
			pcpElement.setHeight(h, iLU);
			pcpElement.setWidth(w, iLU);
			pcpElement.setDepth(d, iLU);


		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new ParsingException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new ParsingException("could not parse coordinate values");
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof PointCloudGrasping))
				return false;

			PointCloudGrasping other = (PointCloudGrasping) obj;

			return other.getConvexhull().equals(convhull)
					&& other.getGraspPoint().equals(grasppnt)
					&& other.getLabel().equals(label)
            		&& other.getHeight(iLU) == getHeight(iLU)
            		&& other.getDepth(iLU) == getDepth(iLU)
            		&& other.getWidth(iLU) == getWidth(iLU);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append(getBaseTag() + "; ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("label: " + label + "; ");
		strb.append("height=" + height + "; ");
		strb.append("width=" + width + "; ");
		strb.append("depth=" + depth + "; ");
		strb.append("grasppoint: " + grasppnt.getX(iLU) + ", "
				+ grasppnt.getY(iLU) + ", " + grasppnt.getZ(iLU) + "; ");
		strb.append("point_cloud: " + convhull.toString() + ";");
		return strb.toString();
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getLabel() {
		return label;
	}

	public void setConvexhull(PointCloud in_cloud) {
		this.convhull = in_cloud;
	}

	public PointCloud getConvexhull() {
		return convhull;
	}

	public void setGraspPoint(double x, double y, double z) {
		this.grasppnt = new Point3D(x, y, z, iLU);
	}

	public Point3D getGraspPoint() {
		return grasppnt;
	}
	
	public double getWidth(LengthUnit unit) {
		return UnitConverter.convert(width, iLU, unit);
	}

	public void setWidth(double width, LengthUnit unit) {
		this.width = UnitConverter.convert(width, unit, iLU);
	}

	public double getHeight(LengthUnit unit) {
		return UnitConverter.convert(height, iLU, unit);
	}

	public void setHeight(double height, LengthUnit unit) {
		this.height = UnitConverter.convert(height, unit, iLU);
	}

	public double getDepth(LengthUnit unit) {
		return UnitConverter.convert(depth, iLU, unit);
	}

	public void setDepth(double depth, LengthUnit unit) {
		this.depth = UnitConverter.convert(depth, unit, iLU);
	}

}
