/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * ObjectLocationDataTest.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/object/ObjectLocationData.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ObjectLocationDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		ObjectLocationData original;

		original.setClassLabel(string("smacks"));

		vector<pair<int, int> > poly;
		poly.push_back(pair<int,int>(0,0));
		poly.push_back(pair<int,int>(0,2));
		poly.push_back(pair<int,int>(2,0));

		ObjectLocationData::Hypothesis originalHyp;
		originalHyp.setGenerator("test");
		originalHyp.setTimestamp(Timestamp(253543797,253543712));
		originalHyp.setReliability(0.1);
		originalHyp.setPolygon(poly);
		original.addHypothesis(originalHyp);
		ObjectLocationData::Hypothesis originalHyp2;
		originalHyp2.setGenerator("test2");
		originalHyp2.setTimestamp(Timestamp(253543797,253543712));
		originalHyp2.setReliability(0.1);
		originalHyp2.setPolygon(poly);
		original.addHypothesis(originalHyp2);

		Location originalLoc(ObjectLocationData::getTemplate(), ObjectLocationData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		ObjectLocationData parsed = extract<ObjectLocationData>(originalLoc);
		BOOST_REQUIRE_EQUAL(parsed.getLocationHypotheses().size(), original.getLocationHypotheses().size());

		ObjectLocationData::Hypothesis parsedHyp = parsed.getLocationHypotheses()[0];
		ObjectLocationData::Hypothesis parsedHyp2 = parsed.getLocationHypotheses().at(1);

		BOOST_REQUIRE_EQUAL(parsedHyp.getGenerator(), originalHyp.getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp2.getGenerator(), originalHyp2.getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp.getTimestamp(), originalHyp.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsedHyp.getReliability(), originalHyp.getReliability(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getClassLabel(), original.getClassLabel());
		BOOST_REQUIRE_EQUAL(parsedHyp.getSensorType(), originalHyp.getSensorType());
		BOOST_REQUIRE_EQUAL(originalHyp.getPolygon().size(), parsedHyp.getPolygon().size());
		BOOST_REQUIRE_EQUAL(originalHyp.getPolygon()[0].first, parsedHyp.getPolygon()[0].first);
		BOOST_REQUIRE_EQUAL(originalHyp.getPolygon()[1].first, parsedHyp.getPolygon()[1].first);
		BOOST_REQUIRE_EQUAL(originalHyp.getPolygon()[2].first, parsedHyp.getPolygon()[2].first);

		BOOST_REQUIRE(original == parsed);
		parsed.setClassLabel("bla");
		BOOST_REQUIRE(!(original == parsed));

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("ObjectLocationData.xml"), ObjectLocationData::getBaseTag());
	ObjectLocationData parsed = extract<ObjectLocationData>(fileContents);
	ObjectLocationData::Hypothesis parsedHyp = parsed.getLocationHypotheses()[0];

	BOOST_REQUIRE_EQUAL("BoostDetector", parsedHyp.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsedHyp.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsedHyp.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.6, parsedHyp.getReliability(), 0.0001);
	BOOST_REQUIRE_EQUAL("smacks", parsed.getClassLabel());
	BOOST_REQUIRE_EQUAL("SONY_EVI", parsedHyp.getSensorType());
	BOOST_REQUIRE_EQUAL(3, parsedHyp.getPolygon().size());
	BOOST_REQUIRE_EQUAL(0, parsedHyp.getPolygon()[0].first);
	BOOST_REQUIRE_EQUAL(1, parsedHyp.getPolygon()[1].first);
	BOOST_REQUIRE_EQUAL(2, parsedHyp.getPolygon()[2].first);

}

BOOST_AUTO_TEST_SUITE_END()
